'use strict';
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
// The module 'vscode' contains the VS Code extensibility API. Import the module and reference it with the alias vscode in your code below.
const vscode = require("vscode");
// Import whole the REST API generated by Swagger with typescript-node flavour
const swagger = require("./client/api");
const license_1 = require("./client/license");
const ConfigLog4j_1 = require("./utils/ConfigLog4j");
const Utils = require("./utils/utils");
// Imports from our sources
const PaginatedDefectsCompiler_1 = require("./PaginatedDefectsCompiler");
const DEFAULT_BASE_PATH = 'https://www.kiuwan.com/saas';
const DEFAULT_REST_PATH = 'https://api.kiuwan.com';
const REST_LICENSE_SUFFIX = '/rest/license';
const KEY_PRIORITY_PREFIX = "com.optimyth.CQM.defectsByPriority";
const KEY_PRIORITY_VERY_HIGH = KEY_PRIORITY_PREFIX + ".Priority 1";
const KEY_PRIORITY_HIGH = KEY_PRIORITY_PREFIX + ".Priority 2";
const KEY_PRIORITY_NORMAL = KEY_PRIORITY_PREFIX + ".Priority 3";
const KEY_PRIORITY_LOW = KEY_PRIORITY_PREFIX + ".Priority 4";
const KEY_PRIORITY_VERY_LOW = KEY_PRIORITY_PREFIX + ".Priority 5";
const VALUE_PRIORITY_1 = "Very High";
const VALUE_PRIORITY_2 = "High";
const VALUE_PRIORITY_3 = "Normal";
const VALUE_PRIORITY_4 = "Low";
const VALUE_PRIORITY_5 = "Very Low";
const REQ_HEADER_DOMAIN_KEY = "X-KW-CORPORATE-DOMAIN-ID";
const log = ConfigLog4j_1.LogFactory.getLogger("k4d.KiuwanService");
function getBasePath() {
    let basePath;
    let serverUrlConfig = vscode.workspace.getConfiguration('kiuwan.connectionSettings.baseServerUrl');
    if (serverUrlConfig.get('active')) {
        basePath = serverUrlConfig.get('URL');
    }
    else {
        basePath = DEFAULT_BASE_PATH;
    }
    return basePath.replace('/\/+$/', '');
}
exports.getBasePath = getBasePath;
function getLicensePath() {
    return getBasePath() + REST_LICENSE_SUFFIX;
}
function getRestPath() {
    let restPath;
    let serverUrlConfig = vscode.workspace.getConfiguration('kiuwan.connectionSettings.baseServerUrl');
    if (serverUrlConfig.get('active')) {
        restPath = serverUrlConfig.get('URL') + '/rest/v1';
    }
    else {
        restPath = DEFAULT_REST_PATH;
    }
    return restPath.replace('/\/+$/', '');
}
function getUsername() {
    let credentialsConfig = vscode.workspace.getConfiguration('kiuwan.connectionSettings.credentials');
    return credentialsConfig.get('kiuwanUser');
}
function getPassword() {
    let credentialsConfig = vscode.workspace.getConfiguration('kiuwan.connectionSettings.credentials');
    let potentiallyEncryptedPassword = credentialsConfig.get('password');
    return Utils.decrypt(potentiallyEncryptedPassword);
}
function getSSODomain() {
    let ssoCredentialsConfig = vscode.workspace.getConfiguration('kiuwan.connectionSettings.credentials.sso');
    return ssoCredentialsConfig.get('domain');
}
class DefaultAuth extends swagger.VoidAuth {
    applyToRequest(requestOptions) {
        commonApplyToRequest(requestOptions);
        super.applyToRequest(requestOptions);
    }
}
class SelfSignedCertAuth {
    applyToRequest(requestOptions) {
        commonApplyToRequest(requestOptions);
        requestOptions.rejectUnauthorized = false;
    }
}
function commonApplyToRequest(requestOptions) {
    let domain = getSSODomain();
    requestOptions.headers[REQ_HEADER_DOMAIN_KEY] = domain;
    requestOptions.timeout = 120000; // in ms == 2 minutes
}
class KiuwanService {
    static getDefaultAuthentication() {
        let basePath = getBasePath();
        if (basePath.indexOf("www.kiuwan.com") !== -1) {
            return KiuwanService.DEFAULT_AUTH;
        }
        else {
            return KiuwanService.SELF_SIGNED_CERT_AUTH;
        }
    }
    /*
     * LICENSE API WRAPPER METHODS - Facilitates using Kiuwan License Client statically
     */
    static getKiuwanLicenseApi() {
        let api = new license_1.KiuwanLicenseApi(getUsername(), getPassword(), getLicensePath());
        api.setDefaultAuthentication(this.getDefaultAuthentication());
        return api;
    }
    /*
     * SWAGGER CLIENT WRAPPER METHODS - Facilitates getting API classes leveraging connection settings injection
     */
    static getActionPlanApi() {
        let api = new swagger.ActionPlanApi(getUsername(), getPassword(), getRestPath());
        api.setDefaultAuthentication(this.getDefaultAuthentication());
        return api;
    }
    static getAnalysesApi() {
        let api = new swagger.AnalysesApi(getUsername(), getPassword(), getRestPath());
        api.setDefaultAuthentication(this.getDefaultAuthentication());
        return api;
    }
    static getApplicationApi() {
        let api = new swagger.ApplicationApi(getUsername(), getPassword(), getRestPath());
        api.setDefaultAuthentication(this.getDefaultAuthentication());
        return api;
    }
    static getArchitectureApi() {
        let api = new swagger.ArchitectureApi(getUsername(), getPassword(), getRestPath());
        api.setDefaultAuthentication(this.getDefaultAuthentication());
        return api;
    }
    static getAuditApi() {
        let api = new swagger.AuditApi(getUsername(), getPassword(), getRestPath());
        api.setDefaultAuthentication(this.getDefaultAuthentication());
        return api;
    }
    static getDefectApi() {
        let api = new swagger.DefectApi(getUsername(), getPassword(), getRestPath());
        api.setDefaultAuthentication(this.getDefaultAuthentication());
        return api;
    }
    static getDeliveryApi() {
        let api = new swagger.DeliveryApi(getUsername(), getPassword(), getRestPath());
        api.setDefaultAuthentication(this.getDefaultAuthentication());
        return api;
    }
    static getDocumentationApi() {
        let api = new swagger.DocumentationApi(getUsername(), getPassword(), getRestPath());
        api.setDefaultAuthentication(this.getDefaultAuthentication());
        return api;
    }
    static getGlobalStatsApi() {
        let api = new swagger.GlobalStatsApi(getUsername(), getPassword(), getRestPath());
        api.setDefaultAuthentication(this.getDefaultAuthentication());
        return api;
    }
    static getInformationApi() {
        let api = new swagger.InformationApi(getUsername(), getPassword(), getRestPath());
        api.setDefaultAuthentication(this.getDefaultAuthentication());
        return api;
    }
    static getLanguagesApi() {
        let api = new swagger.LanguagesApi(getUsername(), getPassword(), getRestPath());
        api.setDefaultAuthentication(this.getDefaultAuthentication());
        return api;
    }
    static getManagementApi() {
        let api = new swagger.ManagementApi(getUsername(), getPassword(), getRestPath());
        api.setDefaultAuthentication(this.getDefaultAuthentication());
        return api;
    }
    static getPortfolioApi() {
        let api = new swagger.PortfolioApi(getUsername(), getPassword(), getRestPath());
        api.setDefaultAuthentication(this.getDefaultAuthentication());
        return api;
    }
    static getSecurityApi() {
        let api = new swagger.SecurityApi(getUsername(), getPassword(), getRestPath());
        api.setDefaultAuthentication(this.getDefaultAuthentication());
        return api;
    }
    static getUserApi() {
        let api = new swagger.UserApi(getUsername(), getPassword(), getRestPath());
        api.setDefaultAuthentication(this.getDefaultAuthentication());
        return api;
    }
    static getUserGroupApi() {
        let api = new swagger.UserGroupApi(getUsername(), getPassword(), getRestPath());
        api.setDefaultAuthentication(this.getDefaultAuthentication());
        return api;
    }
    /*
     * DAO METHODS - Retrieve requested data
     */
    static getLastAnalysisDefects(application, characteristics, languages, priorities, fileContains, orderBy, asc, limit) {
        return __awaiter(this, void 0, void 0, function* () {
            let compiler = new PaginatedDefectsCompiler_1.PaginatedDefectsCompiler();
            compiler.setPageFunction((page, count) => __awaiter(this, void 0, void 0, function* () {
                let result = yield this.getApplicationApi().getDefects(application, characteristics, languages, priorities, fileContains, orderBy, asc, page, count);
                return result.body.defects;
            }));
            try {
                return compiler.get(limit);
            }
            catch (error) {
                vscode.window.showErrorMessage(error.message, 'OK');
            }
        });
    }
    static getActionPlanDefects(application, actionPlan, characteristics, languages, priorities, fileContains, orderBy, asc, limit) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                let result = yield this.getActionPlanApi().getActionPlanPendingDefects(application, actionPlan, undefined, undefined, orderBy, asc, limit, characteristics, languages, priorities, fileContains);
                return result.body.pendingDefects;
            }
            catch (error) {
                vscode.window.showErrorMessage(error.message, 'OK');
            }
        });
    }
    static getAuditDefects(deliveryCode, characteristics, languages, priorities, fileContains, orderBy, asc, limit) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                let result = yield this.getAuditApi().getAuditDefects(deliveryCode, characteristics, languages, priorities, fileContains, orderBy, asc, limit);
                return result.body;
            }
            catch (error) {
                vscode.window.showErrorMessage(error.message, 'OK');
            }
        });
    }
    static getDeliveryDefects(deliveryCode, characteristics, languages, priorities, fileContains, orderBy, asc, limit) {
        let compiler = new PaginatedDefectsCompiler_1.PaginatedDefectsCompiler();
        compiler.setPageFunction((page, count) => __awaiter(this, void 0, void 0, function* () {
            let result = yield this.getAnalysesApi().getAnalysisDefects(deliveryCode, characteristics, languages, priorities, fileContains, orderBy, asc, page, count);
            return result.body.defects;
        }));
        try {
            return compiler.get(limit);
        }
        catch (error) {
            vscode.window.showErrorMessage(error.message, 'OK');
        }
    }
    static getLanguages() {
        return __awaiter(this, void 0, void 0, function* () {
            if (KiuwanService.LANGUAGES) {
                return Promise.resolve(KiuwanService.LANGUAGES);
            }
            try {
                let result = yield this.getLanguagesApi().getLanguages();
                KiuwanService.LANGUAGES = result.body;
                log.info(`Language labels successfully retrieved from Kiuwan`);
                return result.body;
            }
            catch (error) {
                vscode.window.showErrorMessage(error.message, 'OK');
            }
        });
    }
    /*
     * UTILITY METHODS - Operations related with information retrieved from Kiuwan
     */
    static getVeryHighDefectsCount(metrics) {
        if (metrics) {
            let partialCount = metrics[KEY_PRIORITY_VERY_HIGH];
            if (partialCount) {
                return partialCount;
            }
            else {
                return 0;
            }
        }
        else {
            return 0;
        }
    }
    static getHighDefectsCount(metrics) {
        if (metrics) {
            let partialCount = metrics[KEY_PRIORITY_HIGH];
            if (partialCount) {
                return partialCount;
            }
            else {
                return 0;
            }
        }
        else {
            return 0;
        }
    }
    static getNormalDefectsCount(metrics) {
        if (metrics) {
            let partialCount = metrics[KEY_PRIORITY_NORMAL];
            if (partialCount) {
                return partialCount;
            }
            else {
                return 0;
            }
        }
        else {
            return 0;
        }
    }
    static getLowDefectsCount(metrics) {
        if (metrics) {
            let partialCount = metrics[KEY_PRIORITY_LOW];
            if (partialCount) {
                return partialCount;
            }
            else {
                return 0;
            }
        }
        else {
            return 0;
        }
    }
    static getVeryLowDefectsCount(metrics) {
        if (metrics) {
            let partialCount = metrics[KEY_PRIORITY_VERY_LOW];
            if (partialCount) {
                return partialCount;
            }
            else {
                return 0;
            }
        }
        else {
            return 0;
        }
    }
    static getTotalDefectsCount(metrics) {
        let totalDefectsCount = 0;
        totalDefectsCount += this.getVeryHighDefectsCount(metrics);
        totalDefectsCount += this.getHighDefectsCount(metrics);
        totalDefectsCount += this.getNormalDefectsCount(metrics);
        totalDefectsCount += this.getLowDefectsCount(metrics);
        totalDefectsCount += this.getVeryLowDefectsCount(metrics);
        return totalDefectsCount;
    }
    static priorityFromString(priorityString) {
        switch (priorityString) {
            case VALUE_PRIORITY_1:
                return 1;
            case VALUE_PRIORITY_2:
                return 2;
            case VALUE_PRIORITY_3:
                return 3;
            case VALUE_PRIORITY_4:
                return 4;
            case VALUE_PRIORITY_5:
            default:
                return 5;
        }
    }
    static effortAsLabel(effort) {
        if (!effort)
            return effort;
        let minutes = Number(effort);
        if (Number.isNaN(minutes))
            return effort;
        let hours = Math.floor(minutes / 60);
        if (hours == 0) {
            return minutes + 'm';
        }
        else {
            minutes = minutes % 60;
            return hours + 'h ' + (minutes > 0 ? minutes + 'm' : '');
        }
    }
    static getRuleDocumentationURL(modelId, rulecode) {
        let docUrl = `${getBasePath()}/web/documentation?modelId=${modelId}&ruleCode=${rulecode}`;
        let domain = getSSODomain();
        if (domain && domain.trim().length > 0)
            docUrl = docUrl + `&domain=${domain.trim()}`;
        return docUrl;
    }
}
KiuwanService.DEFAULT_AUTH = new DefaultAuth();
KiuwanService.SELF_SIGNED_CERT_AUTH = new SelfSignedCertAuth();
exports.KiuwanService = KiuwanService;
//# sourceMappingURL=KiuwanService.js.map